#include <map>
#include <ctime>
#include <queue>
#include <stack>
#include <cmath>
#include <cstdio>
#include <cstring>
#include <utility>
#include <cstdlib>
#include <iostream>
#include <algorithm>
#define MAXPLANETS 64
#define MAXSHIPS 64
#define criticalTime 1.750
using namespace std;

struct ship {
    int startingPlanet, speed, cost, capacity;
    ship() {}
    ship(int a, int b, int c, int d) : startingPlanet(a), speed(b), cost(c), capacity(d) {}
} a[MAXSHIPS];

struct edge {
    int to, dist;
    edge() {}
    edge(int a, int b) : to(a), dist(b) {}
};

struct flight {
    int start, end, month, win, people;
    flight() {}
    flight(int b, int c, int d, int e, int f) : start(b), end(c), month(d), win(e), people(f) {}
};

struct out {
    int day, time, planet;
    out() {}
    out(int a, int b, int c) : day(a), time(b), planet(c) {}
};

struct state {
    int planet, cost, day, time;
    state() {}
    state(int a, int b, int c, int d) : planet(a), cost(b), day(c), time(d) {}

    bool operator< (const state &other) const {
        if (planet == other.planet) {
            if (cost == other.cost) {
                if (day == other.day) {
                    return time < other.time;
                }
                return day < other.day;
            }
            return cost < other.cost;
        }
        return planet < other.planet;
    }
};

int n, m, f, fail;

double init;
vector<edge> v[MAXPLANETS];
vector<flight> fl[MAXPLANETS][MAXPLANETS];

// .first = time, .second = cost
pair<int, int> travel[MAXPLANETS][MAXPLANETS][MAXSHIPS]; // the cost and time (in hours) of travelling from planet A to planet B with ship C

int used[MAXPLANETS][20]; // is a given hour on a given planet free?
int permanent[MAXPLANETS][20];

inline void set() {
    memset(travel, -1, sizeof(travel));
    for (int i=1; i <= n; ++i) {
        for (int j=0; j < v[i].size(); ++j)
            for (int k=1; k <= m; ++k) {
                travel[i][ v[i][j].to ][ k ] = make_pair( (v[i][j].dist +a[k].speed-1) / a[k].speed, v[i][j].dist * a[k].cost );
            }
    }
}

vector<out> sol[MAXSHIPS];
int ship;

int bestCurrentCost, br, bestBr, found;
out bestChosen[128000];
out chosen[128000];

map<state, bool> mem;
double average, last;

inline void add() {
    sol[ship].resize(bestBr);
    int curPlanet = a[ship].startingPlanet;

    //if (bestBr == 0)
    //    printf("bestbr is 0... \n");

    for (int i=0; i < bestBr; ++i) {
        permanent[curPlanet][ bestChosen[i].time ] = 1;
        sol[ship][i] = bestChosen[i];
        curPlanet = bestChosen[i].planet;
    }
}

inline double getTime() {
    double curTime = clock() / (double) CLOCKS_PER_SEC;

    /*if (curTime > criticalTime) {
        criticalSolution();
    }*/
    return curTime-init;
}

void select(int planet, int cost, int day, int time) {
    //printf("selecting at %d %d %d %d ~~ %d\n", planet, cost, day, time, br);
    if (found == 1 && average+last > getTime()) {
        fail = 1;
        return;
    }
    if (day >= 4) {
        return;
    }

    if (mem.find( state(planet, cost, day, time) ) != mem.end()) {
        return;
    }
    mem[state(planet, cost, day, time)] = true;
    //printf("    selecting at %d %d %d %d ~~ %d\n", planet, cost, day, time, br);

    if (planet == a[ship].startingPlanet && !(day == 0 && time == 4)) { // potentialSolution
        if (!found || cost > bestCurrentCost) {
            found = 1;
            bestCurrentCost = cost;
            bestBr = br;
            //printf("%d %d %d %d\n", day, time, bestBr, cost);
            memcpy(bestChosen, chosen, sizeof(chosen));
        }
    }

    if (time > 4) select(planet, cost, day+1, 4); // just stay there and wait
    if (time >= 16) return;

    if (used[planet][time]) {
        select(planet, cost, day, time+1);
        return;
    }

    for (int nextPlanet = 1; nextPlanet <= n; ++nextPlanet) {
        for (int i=0; i < fl[planet][nextPlanet].size(); ++i) {// looking for a flight
            flight now = fl[planet][nextPlanet][i];

            if (time > now.start) continue;
            if (a[ship].capacity < now.people) continue;
            if (used[planet][now.start]) continue;

            int newTime = now.start + travel[planet][nextPlanet][ship].first;
            if (newTime > now.end) continue;

            used[planet][now.start] = 1;
            chosen[br++] = out(day+1, now.start, nextPlanet);

            select(nextPlanet, cost - travel[planet][nextPlanet][ship].second + now.people * now.win, day, newTime);

            if (fail) return;

            br--;
            used[planet][now.start] = 0;
        }
    }

    for (int nextPlanet = 1; nextPlanet <= n; ++nextPlanet) {// going to random planet
        if (travel[planet][nextPlanet][ship].first < 0) continue;

        int newTime = time + travel[planet][nextPlanet][ship].first;
        if (newTime > 16) continue;

        used[planet][time] = 1;
        chosen[br++] = out(day+1, time, nextPlanet);

        select(nextPlanet, cost - travel[planet][nextPlanet][ship].second, day, newTime);
        if (fail) return;

        used[planet][time] = 0;
        br --;
    }
}

inline void solve() { // (!) sort ships before selecting optimal paths for them
    init = getTime();
    average = ( 1.85 / (double) m );

    for (ship=1; ship <= m; ++ship) {
        //printf("%d %d again\n", ship, m);
        br = 0;
        bestBr = 0;
        bestCurrentCost = 0;
        found = 0;
        fail = 0;
        mem.clear();
        memcpy(used, permanent, sizeof(used));

        int time = 4, cost = 0, planet = a[ship].startingPlanet, day = 0;
        for (int nextPlanet = 1; nextPlanet <= n; ++nextPlanet) {
            for (int i=0; i < fl[planet][nextPlanet].size(); ++i) {// looking for a flight
                flight now = fl[planet][nextPlanet][i];

                if (time > now.start) continue;
                if (a[ship].capacity < now.people) continue;
                if (used[planet][now.start]) continue;

                int newTime = now.start + travel[planet][nextPlanet][ship].first;
                if (newTime > now.end) continue;

                used[planet][now.start] = 1;
                chosen[br++] = out(day+1, now.start, nextPlanet);

                select(nextPlanet, cost - travel[planet][nextPlanet][ship].second + now.people * now.win, day, newTime);

                br--;
                used[planet][now.start] = 0;
            }
        }

        if (used[planet][time] == 0)
        for (int nextPlanet = 1; nextPlanet <= n; ++nextPlanet) {// going to random planet
            if (travel[planet][nextPlanet][ship].first < 0) continue;

            int newTime = time + travel[planet][nextPlanet][ship].first;
            if (newTime > 16) continue;

            used[planet][time] = 1;
            chosen[br++] = out(day+1, time, nextPlanet);

            select(nextPlanet, cost - travel[planet][nextPlanet][ship].second, day, newTime);

            used[planet][time] = 0;
            br --;
        }

        //printf("that was it for this ship\n");
        add();

        last = (double) ship * average;
    }
}

inline void read() {
    scanf("%d%d%d", &n, &m, &f);
    for (int i=1; i <= m; ++i) // getting information about the ships;
        scanf("%d%d%d%d", &a[i].startingPlanet, &a[i].speed, &a[i].cost, &a[i].capacity);

    for (int i=0; i < f; ++i) {
        int from, to, dist, amount;
        scanf("%d%d%d%d", &from, &to, &dist, &amount);
        v[from].push_back( edge(to, dist) );

        for (int j=0; j < amount; ++j) {
            int st, end, month, win, people;
            scanf("%d%d%d%d%d", &st, &end, &month, &win, &people);
            fl[from][to].push_back( flight(st, end, month, win, people) );
        }
    }
}

inline void print() {
    for (int i=1; i <= m; ++i) {
        printf("%d\n", sol[i].size());
        for (int j=0; j < sol[i].size(); ++j)
            printf("%d %d %d\n", sol[i][j].day, sol[i][j].time, sol[i][j].planet);
    }
}

int main() {
    freopen("flights.in", "r", stdin);
    freopen("flights.out", "w", stdout);
    read();
    set();
    solve();
    print();
    return 0;
}
